/*******************************************************************************
 * eMOCHA - electronic Mobile Open-Source Comprehensive Health Application
 * Copyright (c) 2009 Abe Pazos - abe@ccghe.net
 * 
 * This file is part of eMOCHA.
 * 
 * eMOCHA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMOCHA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package net.ccghe.emocha.activities;

import java.util.ArrayList;
import java.util.HashMap;

import net.ccghe.emocha.Constants;
import net.ccghe.emocha.EmochaListItem;
import net.ccghe.emocha.R;
import net.ccghe.emocha.model.DBAdapter;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

public class TrainingDocList extends Activity {
	
	ArrayList<HashMap<String,EmochaListItem>> list = new ArrayList<HashMap<String,EmochaListItem>>();	
	
    private ListView pList;
    private int pType;
    private String appendPath = "";
	private ArrayList<String> folders = new ArrayList<String>();

	private ArrayList<HashMap<String, EmochaListItem>> getNames(ArrayList<String> paths) {
		int count = paths.size();

		list.clear();
		folders.clear();
		int pathLength = 0;
		String finalPath = "";
		String newFolder = "";
		// 
		switch (pType) {
		case R.id.ButtonTrainCourses:
			finalPath = Constants.PATH_TRAINING_COURSES + appendPath;
			pathLength = finalPath.length();
			break;
		case R.id.ButtonTrainLectures:
			finalPath = Constants.PATH_TRAINING_LECTURES + appendPath;
			pathLength = finalPath.length();
			break;
		case R.id.ButtonTrainLibrary:
			finalPath = Constants.PATH_TRAINING_LIBRARY + appendPath;
			pathLength = finalPath.length();
		}

		for (int i = 0; i < count; i++) {
			String path = paths.get(i);
			HashMap<String, EmochaListItem> element = null;
			EmochaListItem elementValue = null;
			
			if (path.substring(pathLength + 1).contains("/")) {
				// The file it's within a folder ... so lets show only the
				// folder
				
				newFolder = path.substring(pathLength, path.indexOf("/", pathLength + 1));
				if (folders.contains(newFolder) == false)
					folders.add(folders.size(), newFolder);
			} else {
				elementValue = new EmochaListItem(path.substring(pathLength,
						path.lastIndexOf(".")).replace("_", " "), path, 0);
				element = new HashMap<String, EmochaListItem>();
				element.put("listItem", elementValue);
				list.add(element);
						
			}

		}
		for (int i = 0; i < folders.size(); i++) 
		{
			HashMap<String, EmochaListItem> element = new HashMap<String, EmochaListItem>();
			EmochaListItem elementValue = new EmochaListItem(folders.get(i).replace("_", " "), 
					"?"+folders.get(i),
					1);
			element.put("listItem", elementValue);
			list.add(i, element);
		}
		return list;
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
	super.onCreate(savedInstanceState);

	setContentView(R.layout.training_list);

	Bundle extras = getIntent().getExtras();
	pType = extras.getInt(Constants.TRAINING_TYPE);
	appendPath = extras.getString(Constants.TRAINING_INITIAL_DIRECTORY);

	pList = (ListView) findViewById(R.id.TrainingList);
	
	TextView mH1 = (TextView) findViewById(R.id.header_title);

	switch (pType) {
	case R.id.ButtonTrainCourses:
	    mH1.setText(R.string.label_training_courses);
	    mH1.append(" - "+ appendPath);
	    getNames(DBAdapter.getFilesFiltered(
	    		DBAdapter.FilterType.BY_TRAINING_COURSES, 
	    		appendPath));
	    break;
	case R.id.ButtonTrainLectures:
	    mH1.setText(R.string.label_training_lectures);
	    mH1.append(" - "+ appendPath);
	    getNames(DBAdapter.getFilesFiltered(
	    		DBAdapter.FilterType.BY_TRAINING_LECTURES, 
	    		appendPath));
	    break;
	case R.id.ButtonTrainLibrary:
	    mH1.setText(R.string.label_training_library);
	    mH1.append(" - "+ appendPath);	    
	    getNames(DBAdapter.getFilesFiltered(
	    		DBAdapter.FilterType.BY_TRAINING_LIBRARY, 
	    		appendPath));
	    break;
	}

	SimpleAdapter adapter = new SimpleAdapter(
   		 this, list, R.layout.listitemlayout, new String[] { "listItem","listItem"},
   		 new int[] { R.id.itemText, R.id.itemIcon } );
      // Bind to our new adapter.
    	adapter.setViewBinder(new ShowViewBinder());
    	pList.setAdapter(adapter);
    	pList.setOnItemClickListener(new AdapterView.OnItemClickListener() {

    	public void onItemClick(AdapterView<?> parent, View v, int tPosition, long id) {
	    	
    		EmochaListItem element = (((HashMap<String, EmochaListItem>)list.get(tPosition)).get("listItem"));
			if (element.internalData.startsWith("?"))
			{
				Intent tIntent = new Intent(getApplicationContext(), TrainingDocList.class);
				tIntent.putExtra(Constants.TRAINING_TYPE, pType);
				tIntent.putExtra(Constants.TRAINING_INITIAL_DIRECTORY, appendPath + element.internalData.substring(1)+"/");
				startActivity(tIntent);
				return;
			}

	    Intent tIntent = null;
		switch (pType) {
		case R.id.ButtonTrainCourses:
		    tIntent = new Intent(getApplicationContext(), TrainingThumb.class);
		    break;
		case R.id.ButtonTrainLectures:
		    tIntent = new Intent(getApplicationContext(), TrainingThumb.class);
		    break;
		case R.id.ButtonTrainLibrary:
		    tIntent = new Intent(getApplicationContext(), HtmlBookViewActivity.class);
		    break;
		}

		tIntent.putExtra(Constants.DOC_NAME, element.viewedData);
	    tIntent.putExtra(Constants.DOC_ID, element.internalData);
	    startActivity(tIntent);

	    }
	});

    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onStop() {
	super.onStop();
    }
    

			
           
       }

class ShowViewBinder implements SimpleAdapter.ViewBinder {

    boolean retval = false;

	public boolean setViewValue(View view, Object data,
			String textRepresentation) {
     if (data == null)
    	 return false;
     EmochaListItem dato = (EmochaListItem)data;
     if (view.getId() == R.id.itemText){

         TextView tv = (TextView) view;
         tv.setText(dato.viewedData); 
         retval = true;
     }else if (view.getId() == R.id.itemIcon)
     {
         int status = dato.state;
         ImageView imageView = (ImageView) view;
         //Log.i("EMOCHA" , "status " + status );
         if(status == 0){
        	 imageView.setImageResource(0);
         }else
         {
        	 imageView.setImageResource(R.drawable.more_arrows);
         }
//         imageView.invalidate();
         retval = true;
     }

     return retval;
	}
}

